#include "MDEv2718.hh"
#include "MDEv1290.hh"
#include "MDEv1731.hh"
#include "MDEv1724.hh"
#include "MDEv830.hh"
#include "MDEv977.hh"

#include "MiceDAQMessanger.hh"
#include "MiceDAQRunStats.hh"
#include "MiceDAQSpillStats.hh"
#include "equipmentList_common.hh"

MiceDAQRunStats   runStats   = *MiceDAQRunStats::Instance();
MiceDAQSpillStats spillStats = *MiceDAQSpillStats::Instance();

void dumpEventV1724(int* dataPtr);
int dumpEventV1290(int* dataPtr);
int nbStored;

void dumpEventV1724_ZS(int* dataPtr) {

  int good = (*dataPtr & 0xF0000000) >>28;
  int evSize = *dataPtr & 0xFFFFFFF;
  double length = double(evSize - 4)/8.;
  dataPtr++;

  int geo = (*dataPtr & 0xF8000000) >> 27;
  int ZS = (*dataPtr & 0x7000000) >> 24;
  int chMask = *dataPtr & 0xff;
  int pattern = (*dataPtr & 0xFFFF00) >> 8;
  dataPtr++;

  int eventCount = *dataPtr & 0xFFFFFF;
  dataPtr++;
  int ttt = *dataPtr;
  dataPtr++;


  std::cout << std::endl << good << "  size = " << evSize << "  length = " << length << std::endl;
  std::cout << "geo: " << geo << "   ZS: " << ZS << "   pattern: " << pattern 
            << "   chmask: "<< std::hex << chMask << std::dec <<std::endl;
  std::cout << "eventCount: " << eventCount << "   ttt: " << ttt << std::endl;

  for(int ich=0;ich<8;ich++) {
    int ch, l;
    std::cout << "ch header: " << *dataPtr << std::endl;
    ch = (*dataPtr & 0xfff000) >> 12;
    l = *dataPtr & 0xfff;
    std::cout << "ch: " << ch << "  l: " << l << std::endl;
    dataPtr++;
    for(int i=0;i<l;i++) {
      /*
      std::cout << ((*dataPtr & 0xFF000000) >> 24) << " "
                << ((*dataPtr & 0xFF0000) >> 16) << " "
                << ((*dataPtr & 0xFF00) >> 8 )<< " "
                << (*dataPtr & 0xFF) << " ";
      */
      std::cout << ((*dataPtr & 0xFFFF0000) >> 16) << " "
                << (*dataPtr & 0xFFFF) << " ";
      dataPtr++;
    }
    std::cout << std::endl;
  }
}
#include <time.h>
clock_t startm, stopm;
#define START if ( (startm = clock()) == -1) {printf("Error calling clock");exit(1);}
#define STOP if ( (stopm = clock()) == -1) {printf("Error calling clock");exit(1);}
#define PRINTTIME printf( "%6.3f microseconds used by the processor.", ((double)stopm-startm)/(1e6*CLOCKS_PER_SEC));

int main(int argc, char** argv) {

  int nEvents = 303;
  try {
    MiceDAQMessanger::Instance()->setVerbosity(0);

    MDEv2718 controler;
    controler.setParams("BNumber", 0)("BLink", 0);
    if ( !controler.Arm() )
      return 0;

    MDEv977 ioReg;
    ioReg.setParams("GEO", 3)("BaseAddress", 0x11020000);

    if ( !ioReg.ArmTriggerReceiver() )
      return false;

    if ( !ioReg.ArmPartTriggerSelector("GVA1") )
      return false;

    if ( !ioReg.ArmTrailer() )
      return false;

    MDE_Pointer* data_tr = new MDE_Pointer[256];
    ioReg.setDataPtr(data_tr);

//////////////////////////////////////////////////////////////////////////////

    MDEv1724 fadc1;
    //char *test = "blah";
    //bool *test_b;
    fadc1.setParams("GEO", 21)
                   //("BaseAddress", 0x11120000)
                   ("BaseAddress", 0x11140000)
                   ("PostTriggerOffset", 42)
                   ("BlockTransfEventNum", 512)
                   ("ChannelMask", 0xffff)
                   ("TriggerOverlapping", 1)
                   ("WordsPerEvent", 64)
                   ("UseSoftwareTrigger", 0)
                   ("UseExternalTrigger", 1)
                   ("ZSThreshold", 100);

    if ( !fadc1.Arm() )
      return 0;

    int memSizeFadc = fadc1.getMaxMemUsed(nEvents);
    MDE_Pointer *data_fadc = new MDE_Pointer[memSizeFadc/4];
    fadc1.setDataPtr(data_fadc);
    std::cout << "Memory alocated (fADC): " << memSizeFadc << std::endl;

//////////////////////////////////////////////////////////////////////////////

    MDEv1290 tdc;
    tdc.setParams("GEO", 11)
                 ("BaseAddress", 0x11060000)
                 ("ChannelMask", 0xffff)
                 ("UseEventFIFO", 1)
                 ("UseExtendedTTT", 1)
                 ("DoInit", 1)
                 ("TriggerMode", 1)
                 ("WinWidth", 120)
                 ("WinOffset", -28)
                 ("SwMargin", 8)
                 ("RejMargin", 2)
                 ("TTSubtraction", 0)
                 ("DetectionEdge", 2)  //1 only trailing; 2 only leading; 3 trailing and leading.
                 ("TDCHeader", 1)
                 ("LSBCode", 3)
                 ("DeadtimeCode", 0);
    /*
    if ( !tdc.Arm() )
      return 0;

    //int memSizeTdc = tdc.getMaxMemUsed(nEvents);
    int memSizeTdc = 2048;
    MDE_Pointer *data_tdc = new MDE_Pointer[memSizeTdc/4];
    tdc.setDataPtr(data_tdc);
    std::cout << "Memory alocated (TDC): " << memSizeTdc << std::endl;
    */
//////////////////////////////////////////////////////////////////////////////

    nbStored = 0;
    bool done = false;
    int duration = 2;
    time_t t1=time(NULL), t2;
    while (!done) {
      t2=time(NULL);
      if ( (t2-t1) > duration ) done = true;
      if ( ioReg.EventArrivedTriggerReceiver() ){
        ioReg.ReadEventTriggerReceiver();
        if ( ioReg.getEventType() == START_OF_BURST ) {
          std::cout << "\n---------------- \n"  << "time: " << t2-t1 << "s" << std::endl;
          std::cout << "Event Arrived. Event type is " << ioReg.getEventTypeAsString() << std::endl;
          std::cout << "TDC events: " << tdc.getNEventsStored() << "    "
                    << "fADC events: " << fadc1.getNEventsStored() << std::endl;
          spillStats["SpillNumber"] ++;
          runStats["NumberOfSpills"] ++;
        } else if (ioReg.getEventType() == PHYSICS_EVENT ) {
          std::cout << "Event Arrived. Event type is " << ioReg.getEventTypeAsString() << std::endl;
          std::cout << "TDC events: " << tdc.getNEventsStored() << "    "
                    << "fADC events: " << fadc1.getNEventsStored() << std::endl;
          //nbStored += tdc.ReadEvent();
          nbStored += fadc1.ReadEvent();
          nbStored += ioReg.ReadEventTrailer();
        } else if ( ioReg.getEventType() == END_OF_BURST ){
          std::cout << "Event Arrived. Event type is " << ioReg.getEventTypeAsString() << std::endl;
          std::cout << "TDC events: " << tdc.getNEventsStored() << "    "
                    << "fADC events: " << fadc1.getNEventsStored() << std::endl;
          ioReg.ReadEventTrailer();
        }
      }
    }
    /*
    std::cout << "\nData recorded: " << nbStored/1e3 << " KB\n";

    int* data_tmp = data_fadc;
    for(int j=0;j<nEvents;j++) {
      dumpEventV1724(data_tmp);
      int evSize = (*data_tmp) & 0xFFFFFFF;
      data_tmp += evSize;
    }

    data_tmp = data_tdc;
    for(int j=0;j<nEvents;j++) {
      int nw = dumpEventV1290(data_tmp);
      data_tmp += nw;
    }
    */
    //free(data_tdc);
    free(data_fadc);
    free(data_tr);
    if( !fadc1.DisArm() )
      return 0;

    if ( !controler.DisArm() )
      return 0;

    return 1;
  } catch (MiceDAQException e) {
    std::cout << e.GetLocation() << std::endl;
    std::cout << e.GetDescription() << std::endl;
    return 0;
  }

  return 1;
}



