#include "equipmentList_common.hh"
#include "MDEv2718.hh"
#include "MDEv1290.hh"
#include "MDEv1731.hh"
#include "MDEv1724.hh"
#include "MDEv830.hh"
#include "MDEv977.hh"

#include "MiceDAQMessanger.hh"
#include "MiceDAQRunStats.hh"
#include "MiceDAQSpillStats.hh"

extern int nbStored;
extern MiceDAQSpillStats spillStats;
extern MiceDAQRunStats   runStats;


/////////////////////////////////////  TDC //////////////////////////////////
int dumpEventV1290(int* dataPtr) {
  const short gl_header  = 0x8;
  const short gl_trailer = 0x10;
  const short header     = 0x1;
  const short mesurmrnt  = 0x0;
  const short error      = 0x4;
  const short trailer    = 0x3;
  const short ext_ttt    = 0x11;

  bool done = false;
  int nWords = 0;
  while (!done) {

    short data_type = (*dataPtr & 0xF8000000) >> 27;

    switch (data_type) {
      case header:
        //std::cout << std::setw(2) << nWords << "   header->        tdc: " << ((*dataPtr & 0x3000000) >> 24)
          //        << "   eventId: " << ((*dataPtr & 0xfff000) >> 12)
            //      << "   bunchId: " << (*dataPtr & 0xfff) << std::endl;
      break;

      case gl_header:
        std::cout << std::setw(2) << nWords << "   gl_header->        ev_count: " << ((*dataPtr & 0x7FFFFFE) >> 5)
                  << "   geo: " << (*dataPtr & 0x1f) << std::endl;
        break;

      case mesurmrnt:
        std::cout << std::setw(2) << nWords<< "   measur. ->  ch: " << std::setw(2) << ((*dataPtr & 0x3E00000) >> 21)
                  << "   time: " << (*dataPtr & 0x1FFFFF)*0.025;
        if (*dataPtr & 0x4000000) std::cout << " (t)" << std::endl;
        else  std::cout << " (l)" << std::endl;
        break;

      case error:
        std::cout << std::setw(2) << nWords << "   error ->       tdc: " << ((*dataPtr & 0x3000000) >> 24)
                  << "   errFlag: " << std::hex << (*dataPtr & 0x7fff) << std::dec << std::endl;
        break;

      case ext_ttt:
        std::cout << std::setw(2) << nWords << "   ext_ttt: " << (*dataPtr & 0x7ffffff) << std::endl;
        break;

      case trailer:
        //std::cout << std::setw(2) << nWords << "   trailer ->     tdc: " << ((*dataPtr & 0x3000000) >> 24)
          //        << "   eventId: " << ((*dataPtr & 0xfff000) >> 12)
            //      << "   wd_count: " << (*dataPtr & 0xfff) << std::endl;
        break;

      case gl_trailer:
        std::cout << std::setw(2) << nWords << "   gl_trailer -> gl_wd_count: " << ((*dataPtr & 0x1ffff0) >> 5)
                  << "   geo: " << (*dataPtr & 0x1f) << std::endl;
        done = true;
        break;

      default:
        std::cout << nWords << "unknown data type !!!" << std::endl;
        break;
    }
    dataPtr++;
    nWords++;
  }

  return nWords;
}

bool testV1290(int ba) {

  int nEvents = 2;
  MDEv1290 tdc;
  tdc.setParams("GEO", 11)
               ("BaseAddress", ba)
               ("ChannelMask", 0xffff)
               ("UseEventFIFO", 1)
               ("UseExtendedTTT", 1)
               ("DoInit", 1)
               ("TriggerMode", 1)
               ("WinWidth", 1200)
               ("WinOffset", -28)
               ("SwMargin", 8)
               ("RejMargin", 2)
               ("TTSubtraction", 1)
               ("DetectionEdge", 2)  //1 only trailing; 2 only leading; 3 trailing and leading.
               ("TDCHeader", 1)
               ("LSBCode", 3)
               ("DeadtimeCode", 0);

  if ( !tdc.Arm() )
    return false;

  for(int j=0;j<nEvents;j++) {
    sleep(1);
    tdc.softwareTrigger();
  }

  tdc.getStatusReg();
  std::cout << "NTriggers = " << tdc.getNTiggers() <<
               "  NEventsStored = " << tdc.getNEventsStored() << std::endl;

  int memSize = tdc.getMaxMemUsed(nEvents);
  std::cout << "Memory allocated: " << memSize << std::endl;

  MDE_Pointer* data = (MDE_Pointer*) malloc (memSize);
  tdc.setDataPtr(data);

  nbStored = tdc.ReadEvent();
  spillStats["DataRecorded"] += nbStored;

  int* data_tmp = (int*)data;

  for(int j=0;j<nEvents;j++) {
    int nw = dumpEventV1290(data_tmp);
    data_tmp += nw;
  }

  if ( !tdc.DisArm() )
    return false;

  free(data);

  return true;
}


///////////////////////////////////// IO-Register ////////////////////////////////
bool testV977(int ba, int duration) {

  MiceDAQMessanger::Instance()->setVerbosity(1);
  MDEv977 ioReg;
  ioReg.setParams("GEO", 3)("BaseAddress", ba);

  if ( !ioReg.ArmTriggerReceiver() )
    return false;

  if ( !ioReg.ArmTrailer() )
    return false;


/*
  ioReg.dump();

  ioReg.EventArrivedTriggerReceiver();
  ioReg.dump();

  ioReg.setInput(0x10);
  ioReg.dump();

  ioReg.EventArrivedTriggerReceiver();
  ioReg.dump();
  ioReg.ReadEventTriggerReceiver();
  ioReg.dump();

  ioReg.setInput(0x1);
  ioReg.dump();

  ioReg.ReadEventTriggerReceiver();
  ioReg.dump();

  ioReg.setInput(0x40);
  ioReg.dump();

  ioReg.ReadEventTriggerReceiver();
  ioReg.dump();
*/
  MDE_Pointer* data = (MDE_Pointer*) malloc (1024);
  ioReg.setDataPtr(data);

  bool done = false;
  time_t t1=time(NULL), t2;
  std::cout << "V977 test duration: " << duration << " s"
            << "\nExpected number of spills > " << duration/3 << std::endl;

  while (!done) {
    t2=time(NULL);
    if ( (t2-t1) > duration ) done = true;
    if ( ioReg.EventArrivedTriggerReceiver() ){
      ioReg.ReadEventTriggerReceiver();
      if ( ioReg.getEventType() == START_OF_BURST ) {
        std::cout << "\n---------------- \n"  << "time: " << t2-t1 << "s" << std::endl;
        spillStats["SpillNumber"] ++;
        runStats["NumberOfSpills"] ++;
      }
      std::cout << "Event Arrived. Event type is " << ioReg.getEventTypeAsString() << std::endl;
      ioReg.ReadEventTrailer();
    }
  }

  if (!ioReg.DisArmTriggerReceiver() )
    return false;

  if( !ioReg.DisArmTrailer() )
    return false;

  free(data);

  if (runStats["NumberOfSpills"] < duration/3)
    return false;

  return true;
}


//////////////////////////// Scaler /////////////////////////////////////
void dumpEventV830(int* dataPtr) {
  int geo = (*dataPtr & 0xF8000000) >> 27;
  int nw = (*dataPtr & 0xFC0000) >> 18;
  int ts = (*dataPtr & 0x30000) >> 16;
  int trNum = *dataPtr & 0xFFFF;
  std::cout << std::endl << "geo: " << geo << "  nWords: " << nw << "  ts: " << ts << "  trNum: " << trNum << std::endl;
  dataPtr++;
  for(int w=1;w<nw;w++) {
    int ch = (*dataPtr & 0xF8000000) >> 27;
    int count = *dataPtr & 0x3F00000;
    std::cout << w << "  Ch: " << ch << "   count: " << count << std::endl;
    dataPtr++;
  }
  std::cout << std::endl;
}


bool testV830(int ba) {

  MDEv830 scaler;
  scaler.setParams("GEO", 11)
                  ("BaseAddress", ba)
                  ("ChannelMask", 0xfffff000)
                  ("TriggerMode", 1)
                  ("UseHeader", 1)
                  ("UseNarrow", 1)
                  ("AutoReset", 1);

  if ( !scaler.Arm() )
    return false;

  scaler.softwareTrigger();
  int memSizeScaler = scaler.getMaxMemUsed(1);
  std::cout << "Memory allocated: " << memSizeScaler << std::endl;

  MDE_Pointer* data = (MDE_Pointer*) malloc (memSizeScaler);
  scaler.setDataPtr(data);
  nbStored = scaler.ReadEvent();
  spillStats["DataRecorded"] += nbStored;
  int* data_tmp = (int*)data;
  dumpEventV830(data_tmp);

  free(data);

  if( !scaler.DisArm() );
  return false;

  return true;
}


/////////////////////// Flash ADC V1731 ///////////////////////////////////
void dumpEventV1731(int* dataPtr) {

    int good = (*dataPtr & 0xF0000000) >>28;
    int evSize = *dataPtr & 0xFFFFFFF;
    double length = double(evSize - 4)/8.;
    dataPtr++;

    int geo = (*dataPtr & 0xF8000000) >> 27;
    int ZS = (*dataPtr & 0x7000000) >> 24;
    int chMask = *dataPtr & 0xff;
    int pattern = (*dataPtr & 0xFFFF00) >> 8;
    dataPtr++;

    int eventCount = *dataPtr & 0xFFFFFF;
    dataPtr++;
    int ttt = *dataPtr;
    dataPtr++;


    std::cout << std::endl << good << "  size = " << evSize << "  length = " << length << std::endl;
    std::cout << "geo: " << geo << "   ZS: " << ZS << "   pattern: " << pattern 
              << "   chmask: "<< std::hex << chMask << std::dec <<std::endl;
    std::cout << "eventCount: " << eventCount << "   ttt: " << ttt << std::endl;

    for(int ch=0;ch<8;ch++) {
      std::cout << "ch" << ch << ": ";
      for(int i=0;i<length;i++) {
        std::cout << ((*dataPtr & 0xFF000000) >> 24) << " "
        << ((*dataPtr & 0xFF0000) >> 16) << " "
        << ((*dataPtr & 0xFF00) >> 8 )<< " "
        << (*dataPtr & 0xFF) << " ";

        dataPtr++;
      }
      std::cout << std::endl;
    }
  std::cout << std::endl;
}

bool testV1731(int ba) {

  int nEvents = 32;
  MDEv1731 fadc1;
  fadc1.setParams("GEO", 21)
                 ("BaseAddress", ba)
                 ("PostTriggerOffset", 42)
                 ("BufferOrganizationCode", 0x0A)
                 ("BlockTransfEventNum", 1024)
                 ("ChannelMask", 0xffff)
                 ("TriggerOverlapping", 1)
                 ("WordsPerEvent", 64)
                 ("UseSoftwareTrigger", 1)
                 ("UseExternalTrigger", 0)
                 ("DualEdgeSampling", 0);

  if ( !fadc1.Arm() )
    return false;

  int memSizeFadc1 = fadc1.getMaxMemUsedBLT(nEvents);
  MDE_Pointer* data = (MDE_Pointer*) malloc (memSizeFadc1);
  std::cout << "Memory alocated: " << memSizeFadc1 << "(" << fadc1.getMaxMemUsed(nEvents) << ")" << std::endl;
  fadc1.setDataPtr(data);

  for(int j=0;j<nEvents;j++) {
    usleep(1000);
    fadc1.softwareTrigger();
  }

  nbStored = fadc1.ReadEvent();
  spillStats["DataRecorded"] += nbStored;

  int* data_tmp = (int*)data;
  for(int j=0;j<nEvents;j++) {
    dumpEventV1731(data_tmp);
    int evSize = (*data_tmp) & 0xFFFFFFF;
    data_tmp += evSize;
  }

  free(data);
  if( !fadc1.DisArm() )
    return false;

  return true;
}


/////////////////////// Flash ADC V1724 ///////////////////////////////////
void dumpEventV1724(int* dataPtr) {

    int good = (*dataPtr & 0xF0000000) >>28;
    int evSize = *dataPtr & 0xFFFFFFF;
    double length = double(evSize - 4)/8.;
    dataPtr++;
    int geo = (*dataPtr & 0xF8000000) >> 27;
    int chMask = *dataPtr & 0xff;
    int pattern = (*dataPtr & 0xFFFF00) >> 8;
    dataPtr++;
    int eventCount = *dataPtr & 0xFFFFFF;
    dataPtr++;
    int ttt = *dataPtr;
    dataPtr++;


    std::cout << std::endl << good << "  size = " << evSize << "  length = " << length << std::endl;
    std::cout << "geo: " << geo << "  pattern: " << pattern << "   chmask: "<< std::hex << chMask << std::dec <<std::endl;
    std::cout << "eventCount: " << eventCount << "   ttt: " << ttt << std::endl;

    for(int ch=0;ch<8;ch++) {
      std::cout << "ch" << ch << ": ";
      for(int i=0;i<length;i++) {
        std::cout << ((*dataPtr & 0x3FFF0000) >> 16) << " "
        << (*dataPtr & 0x3FFF) << " ";

        dataPtr++;
      }
      std::cout << std::endl;
    }
  std::cout << std::endl;
}

bool testV1724(int ba) {

  int nEvents = 3;
  MDEv1724 fadc2;
  fadc2.setParams("GEO", 22)
                 //("BaseAddress", 0x110b0000)
                 ("BaseAddress", ba)
                 ("PostTriggerOffset", 42)
                 ("BUfferOrganizationCode", 0x0A)
                 ("BlockTransfEventNum", 0xff)
                 ("ChannelMask", 0xffff)
                 ("TriggerOverlapping", 1)
                 ("WordsPerEvent", 4)
                 ("UseSoftwareTrigger", 1)
                 ("UseExternalTrigger", 0);

  if ( !fadc2.Arm() )
    return false;

  int memSizeFadc2 = fadc2.getMaxMemUsed(nEvents);
  MDE_Pointer* data2 = (MDE_Pointer*) malloc (memSizeFadc2);
  std::cout << "Memory alocated: " << memSizeFadc2 << std::endl;
  fadc2.setDataPtr(data2);

  for(int j=0;j<nEvents;j++) {
    sleep(1);
    fadc2.softwareTrigger();
  }

  nbStored = fadc2.ReadEvent();
  spillStats["DataRecorded"] += nbStored;

  int* data_tmp = (int*)data2;
  for(int j=0;j<nEvents;j++) {
    dumpEventV1724(data_tmp);
    int evSize = (*data_tmp) & 0xFFFFFFF;
    data_tmp += evSize;
  }

  free(data2);
  if( !fadc2.DisArm() )
    return false;

  return true;
}

