#include "MDEv2718.hh"
#include "MDEv1290.hh"
#include "MDEv1731.hh"
#include "MDEv1724.hh"
#include "MDEv830.hh"
#include "MDEv977.hh"

#include "MiceDAQMessanger.hh"
#include "MiceDAQRunStats.hh"
#include "MiceDAQSpillStats.hh"
#include "equipmentList_common.hh"

void dumpEventV1731(int* dataPtr);
void dumpEventV1724(int* dataPtr);
int dumpEventV1290(int* dataPtr);
void dumpEventV830(int* dataPtr);
bool testV830(int ba);
bool testV977(int ba, int duration);
bool testV1290(int ba);
bool testV1731(int ba);
bool testV1724(int ba);

int nbStored;
int nEvents=1;

MiceDAQRunStats   runStats   = *MiceDAQRunStats::Instance();
MiceDAQSpillStats spillStats = *MiceDAQSpillStats::Instance();

int main(int argc, char** argv) {

  try {
    MiceDAQMessanger::Instance()->setVerbosity(0);
    //runStats->getInfo();
    //spillStats->getInfo();

    if (argc!=3) {
      std::cout << "Usage:     equipmentTest  <equipment name>  <base address> \n"
                << "Examples: ./equipmentTest V1290 0x21100000 \n"
                << "          ./equipmentTest V977 21020000 \n"
                << "          ./equipmentTest V1724 0x11120000 \n"
                << "          ./equipmentTest V1731 11140000 \n"
                << "          ./equipmentTest V830 11030000" << std::endl;
      return 0;
    }

    std::string eqName(argv[1]);
    int baseAddress = getBA(argv[2]);

    spillStats.setParams("ParticleTriggers", 0)("DataRecorded", 0)("ErrorFlag", 0);

    MDEv2718 controler;
    controler.setParams("BNumber", 0)("BLink", 0);
    if ( !controler.Arm() )
      return 0;

    bool testOK = false;
    if (eqName == "V977")  testOK = testV977(baseAddress, 10);
    if (eqName == "V830")  testOK = testV830(baseAddress);
    if (eqName == "V1290") testOK = testV1290(baseAddress);
    if (eqName == "V1724") testOK = testV1724(baseAddress);
    if (eqName == "V1731") testOK = testV1731(baseAddress);

    if( !testOK )
      return 0;

    if ( !controler.DisArm() )
      return 0;

    if (eqName == "V977") runStats.getInfo();
    else spillStats.getInfo();

  } catch(MiceDAQException e) {
    std::cout << e.GetLocation() << std::endl;
    std::cout << e.GetDescription() << std::endl;
    return 0;
  }

  return 1;
}








