#include "DBBDataProcessor.hh"
#include "DBBSpillData.hh"

#define MAX(a, b) (((a) > (b)) ? (a) : (b))
using namespace MAUS;

int DBBDataProcessor::Process(MDdataContainer* aFragPtr) {
  // Cast the argument to structure it points to.
  // This process should be called only with MDfragmentDBB argument.
  if ( typeid(*aFragPtr) != typeid(MDfragmentDBB) )
    return CastError;

  MDfragmentDBB* xDBBFragment = static_cast<MDfragmentDBB*>(aFragPtr);
//   xDBBFragment->Dump();
  int xLdc, xGeo, xEquip;
  string xDetector = "emr";

  //Json::Value pBoardDoc;
  //Json::Value xDBBHit;

  if ( !xDBBFragment->IsValid() )
    return GenericError;

  DBBSpillData xDBBSpill;
  xDBBSpill.SetDetector(xDetector);
  xLdc = this->GetLdcId();
  xDBBSpill.SetLdcId(xLdc);
  xDBBSpill.SetTimeStamp(this->GetTimeStamp());
  xGeo = xDBBFragment->GetGeo();
  xDBBSpill.SetDBBId(xGeo);
  xDBBSpill.SetSpillNumber(xDBBFragment->GetSpillNumber());
  xDBBSpill.SetSpillWidth(xDBBFragment->GetSpillWidth());
  xDBBSpill.SetTriggerCount(xDBBFragment->GetTriggerCount());
  xDBBSpill.SetHitCount(xDBBFragment->GetHitCount());

  DBBHitsArray hits;
  DBBHitsArray triggers;

  // Put static data into the Json
//   pBoardDoc["detector"]      = xDetector;
//   pBoardDoc["ldc_id"]       = xLdc = this->GetLdcId();
//   pBoardDoc["equip_type"] = xEquip = this->GetEquipmentType();
//   pBoardDoc["time_stamp"]          = this->GetTimeStamp();
//   pBoardDoc["spill_number"]        = xDBBFragment->GetSpillNumber();
//   pBoardDoc["spill_width"]         = xDBBFragment->GetSpillWidth();
//   pBoardDoc["geo"]          = xGeo = xDBBFragment->GetGeo();
//   pBoardDoc["trigger_count"]       = xDBBFragment->GetTriggerCount();
//   pBoardDoc["hit_count"]           = xDBBFragment->GetHitCount();

  // Loop over all the channels
  for (unsigned int xCh=0; xCh<DBB_NCHANNELS; xCh++) {
    // Get the number of leading edge hits.
    int xHitsL = xDBBFragment->GetNLeadingEdgeHits(xCh);
    // Get the number of trailing edge hits.
    int xHitsT = xDBBFragment->GetNTrailingEdgeHits(xCh);

    // By definition the number of the leading edge hits has to be equal
    // to the number of the trailing edge hits but some times this is not true.
    int xMaxHits = MAX(xHitsL, xHitsT);
/*    if (xMaxHits) {*/
    int xLT, xTT;  // Lead and Trail times
    for (int xHit=0; xHit<xMaxHits; xHit++) {
      DBBHit *hit= new DBBHit();
      hit->SetLDC(xLdc);
      hit->SetGEO(xGeo);
      hit->SetChannel(xCh);
//       hit->SetLTimes(xDBBFragment->GetLeadingTimes(xCh));
//       hit->SetTTimes(xDBBFragment->GetTrailingTimes(xCh));

      if (xHit < xHitsL) {
        xLT = xDBBFragment->GetHitMeasurement(xHit,  // Hit ID
                                             xCh,  // Channel ID
                                             'l');
      } else {
        // If no trailing edge hit set the value og time to -99.
        xLT = 0;
      }

      if (xHit < xHitsT) {
        xTT = xDBBFragment->GetHitMeasurement(xHit,  // Hit ID
                                              xCh,  // Channel ID
                                              't');
      } else {
        // If no trailing edge hit set the value og time to -99.
        xTT = 0;
      }
      hit->SetLTime(xLT);
      hit->SetTTime(xTT);
      if (_debug_mode) hit->print();

      if (xCh == DBB_TRIGGER_CHANNEL)
        triggers.push_back(hit);
      else
        hits.push_back(hit);
    }
  }
  xDBBSpill.SetDBBHitsArray(hits);
  xDBBSpill.SetDBBTriggersArray(triggers);
  if (_debug_mode) xDBBSpill.print();

  _spill->push_back(xDBBSpill);
//  _spill->SetDBBSpill(xDBBSpill);
//  ( *_docSpill )[xDetector][_equipment].append(pBoardDoc);
//   _daqxDBBSpill.GetEMRDaq().SetDBBSpill(xDBBSpill);
  return OK;
}

